import io.undertow.server.HttpHandler;
import io.undertow.server.HttpServerExchange;
import io.undertow.websockets.core.UTF8Output;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.nio.ByteBuffer;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;

import org.xnio.Pooled;
import org.xnio.channels.StreamSourceChannel;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.util.concurrent.FutureCallback;
import com.google.common.util.concurrent.Futures;
import com.google.common.util.concurrent.ListenableFuture;
import com.google.common.util.concurrent.ListeningExecutorService;
import com.google.common.util.concurrent.MoreExecutors;

public class SignInHandler implements HttpHandler {

	/**
	 * Employer - sign-in state
	 */
	public static class SignInState {
		public String userName;

		public String password;

	}

	private static SignInState lastRequestedState;
	protected static ObjectMapper objectMapper = new ObjectMapper();
	{
		objectMapper.setSerializationInclusion(Include.NON_NULL);
		objectMapper
				.enable(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT);
	}

	public void handleRequest(HttpServerExchange exchange) throws Exception {
		if (exchange.isInIoThread()) {
			exchange.dispatch(this);
			return;
		}

		readRequest(exchange);
	}

	public static interface ReadHandler {

		public abstract void complete(String string);

		public abstract void failed(Exception e);
	};

	public void readString(final Pooled<ByteBuffer> resource,
			final StreamSourceChannel channel, ReadHandler handler) {
		UTF8Output string = new UTF8Output();

		ByteBuffer buffer = resource.getResource();
		try {
			int r = 0;
			do {
				r = channel.read(buffer);
				if (r == 0) {
					return;
				} else if (r == -1) {
					handler.complete(string.extract());
				} else {
					buffer.flip();
					string.write(buffer);
				}
			} while (r > 0);
		} catch (IOException e) {
			handler.failed(e);
		} finally {
			resource.free();
		}
	}

	private void readRequest(final HttpServerExchange exchange) {
		ReadHandler handler = new ReadHandler() {

			@Override
			public void complete(String inputString) {
				try {

					lastRequestedState = objectMapper.readValue(inputString,
							SignInState.class);
					asyncHttpFetch(exchange);

					return;
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (Exception e) {
					e.printStackTrace();
				}

			}

			@Override
			public void failed(Exception e) {
				e.printStackTrace();
			}

		};
		readString(exchange.getConnection().getBufferPool().allocate(),
				exchange.getRequestChannel(), handler);
	}

	protected static void sendResponse(HttpServerExchange exchange,
			Object response) {
		try {
			exchange.getResponseSender().send(
					objectMapper.writeValueAsString(response));
		} catch (JsonProcessingException e1) {
			e1.printStackTrace();
		}
	}

	public static class Response {
		private InputStream body;
		private HttpServerExchange exchange;

		public Response(HttpServerExchange exchange, InputStream body) {
			this.body = body;
			this.exchange = exchange;
		}

		public InputStream getBody() {
			return body;
		}
	}

	public static class Request implements Callable<Response> {
		private URL url;
		HttpServerExchange exchange;

		public Request(final HttpServerExchange exchange, final URL url) {
			this.url = url;
			this.exchange = exchange;
		}

		@Override
		public Response call() throws Exception {
			Response res = new Response(exchange, url.openStream());

			return res;
		}
	}

	private void asyncHttpFetch(final HttpServerExchange exchange) {
		exchange.dispatch();
		try {
			ListeningExecutorService service = MoreExecutors
					.listeningDecorator(Executors.newFixedThreadPool(1));
			ListenableFuture<Response> resr = service
					.submit(new Callable<Response>() {
						public Response call() {
							try {
								return new Response(exchange, new URL(
										"http://google.com").openStream());
							} catch (Exception e) {
								e.printStackTrace();
							}

							return null;
						}
					});

			Futures.addCallback(resr, new FutureCallback<Response>() {
				public void onSuccess(Response employerCredentials) {
					sendResponse(exchange, lastRequestedState);
				}

				@Override
				public void onFailure(Throwable e) {
					e.printStackTrace();
				}
			});
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}